@@ -31,9 +31,6 @@
 #include "alsactl.h"
 
 
-#define ARRAY_SIZE(a) (sizeof (a) / sizeof (a)[0])
-
-
 static char *id_str(snd_ctl_elem_id_t *id)
 {
 	static char str[128];
@@ -185,11 +182,11 @@ static int add_tlv_comments(snd_ctl_t *handle, snd_ctl_elem_id_t *id,
 		char *s = tlv_to_str(tlv);
 		if (s) {
 			err = snd_config_string_add(comment, "tlv", s);
+			free(s);
 			if (err < 0) {
 				error("snd_config_string_add: %s", snd_strerror(err));
 				return err;
 			}
-			free(s);
 		}
 	}
 
@@ -847,6 +844,49 @@ static int get_comment_range(snd_config_t *n, int ctype,
 	return 0;
 }
 
+struct string_array {
+	unsigned int count;
+	const char **strings;
+};
+
+static int get_comment_items(snd_config_t *n, struct string_array *items)
+{
+	snd_config_iterator_t it, next;
+	unsigned int i;
+	int err;
+
+	snd_config_for_each(it, next, n) {
+		snd_config_t *item = snd_config_iterator_entry(it);
+		const char *id;
+		unsigned int numid;
+
+		if (snd_config_get_id(item, &id) < 0)
+			return -EINVAL;
+		numid = atoi(id);
+		if (numid > 999999)
+			return -EINVAL;
+
+		if (numid >= items->count) {
+			const char **strings = realloc(items->strings, (numid + 1) * sizeof(char *));
+			if (!strings)
+				return -ENOMEM;
+			for (i = items->count; i < numid + 1; ++i)
+				strings[i] = NULL;
+			items->count = numid + 1;
+			items->strings = strings;
+		}
+		err = snd_config_get_string(item, &items->strings[numid]);
+		if (err < 0)
+			return err;
+	}
+
+	for (i = 0; i < items->count; ++i)
+		if (!items->strings[i])
+			return -EINVAL;
+
+	return 0;
+}
+
 static int add_user_control(snd_ctl_t *handle, snd_ctl_elem_info_t *info, snd_config_t *conf)
 {
 	snd_ctl_elem_id_t *id;
@@ -854,12 +894,15 @@ static int add_user_control(snd_ctl_t *handle, snd_ctl_elem_info_t *info, snd_co
 	long imin, imax, istep;
 	snd_ctl_elem_type_t ctype;
 	unsigned int count;
+	struct string_array enum_items;
 	int err;
 	unsigned int *tlv;
 
 	imin = imax = istep = 0;
 	count = 0;
 	ctype = SND_CTL_ELEM_TYPE_NONE;
+	enum_items.count = 0;
+	enum_items.strings = NULL;
 	tlv = NULL;
 	snd_config_for_each(i, next, conf) {
 		snd_config_t *n = snd_config_iterator_entry(i);
@@ -869,31 +912,39 @@ static int add_user_control(snd_ctl_t *handle, snd_ctl_elem_info_t *info, snd_co
 		if (strcmp(id, "type") == 0) {
 			err = get_comment_type(n);
 			if (err < 0)
-				return err;
+				goto error;
 			ctype = err;
 			continue;
 		}
 		if (strcmp(id, "range") == 0) {
 			err = get_comment_range(n, ctype, &imin, &imax, &istep);
 			if (err < 0)
-				return err;
+				goto error;
 			continue;
 		}
 		if (strcmp(id, "count") == 0) {
 			long v;
 			if ((err = snd_config_get_integer(n, &v)) < 0)
-				return err;
+				goto error;
 			count = v;
 			continue;
 		}
+		if (strcmp(id, "item") == 0) {
+			err = get_comment_items(n, &enum_items);
+			if (err < 0)
+				goto error;
+			continue;
+		}
 		if (strcmp(id, "tlv") == 0) {
 			const char *s;
 			if ((err = snd_config_get_string(n, &s)) < 0)
-				return -EINVAL;
+				goto error;
 			if (tlv)
 				free(tlv);
-			if ((tlv = str_to_tlv(s)) == NULL)
-				return -EINVAL;
+			if ((tlv = str_to_tlv(s)) == NULL) {
+				err = -EINVAL;
+				goto error;
+			}
 			continue;
 		}
 	}
@@ -904,8 +955,10 @@ static int add_user_control(snd_ctl_t *handle, snd_ctl_elem_info_t *info, snd_co
 		count = 1;
 	switch (ctype) {
 	case SND_CTL_ELEM_TYPE_INTEGER:
-		if (imin > imax || istep > imax - imin)
-			return -EINVAL;
+		if (imin > imax || istep > imax - imin) {
+			err = -EINVAL;
+			goto error;
+		}
 		err = snd_ctl_elem_add_integer(handle, id, count, imin, imax, istep);
 		if (err < 0)
 			goto error;
@@ -915,6 +968,10 @@ static int add_user_control(snd_ctl_t *handle, snd_ctl_elem_info_t *info, snd_co
 	case SND_CTL_ELEM_TYPE_BOOLEAN:
 		err = snd_ctl_elem_add_boolean(handle, id, count);
 		break;
+	case SND_CTL_ELEM_TYPE_ENUMERATED:
+		err = snd_ctl_elem_add_enumerated(handle, id, count,
+						  enum_items.count, enum_items.strings);
+		break;
 	case SND_CTL_ELEM_TYPE_IEC958:
 		err = snd_ctl_elem_add_iec958(handle, id);
 		break;
@@ -925,6 +982,7 @@ static int add_user_control(snd_ctl_t *handle, snd_ctl_elem_info_t *info, snd_co
 
  error:
 	free(tlv);
+	free(enum_items.strings);
 	if (err < 0)
 		return err;
 	return snd_ctl_elem_info(handle, info);
@@ -1485,6 +1543,7 @@ int save_state(const char *file, const char *cardname)
 	snd_input_t *in;
 	snd_output_t *out;
 	int stdio;
+	char *nfile = NULL;
 
 	err = snd_config_top(&config);
 	if (err < 0) {
@@ -1492,13 +1551,24 @@ int save_state(const char *file, const char *cardname)
 		return err;
 	}
 	stdio = !strcmp(file, "-");
+	if (!stdio) {
+		nfile = malloc(strlen(file) + 5);
+		if (nfile == NULL) {
+			error("No enough memory...");
+			goto out;
+		}
+		strcpy(nfile, file);
+		strcat(nfile, ".new");
+		if (state_lock(file, 1, 10) != 0)
+			goto out;
+	}
 	if (!stdio && (err = snd_input_stdio_open(&in, file, "r")) >= 0) {
 		err = snd_config_load(config, in);
 		snd_input_close(in);
 #if 0
 		if (err < 0) {
 			error("snd_config_load error: %s", snd_strerror(err));
-			return err;
+			goto out;
 		}
 #endif
 	}
@@ -1514,17 +1584,19 @@ int save_state(const char *file, const char *cardname)
 			if (card < 0) {
 				if (first) {
 					if (ignore_nocards) {
-						return 0;
+						err = 0;
+						goto out;
 					} else {
 						error("No soundcards found...");
-						return -ENODEV;
+						err = -ENODEV;
+						goto out;
 					}
 				}
 				break;
 			}
 			first = 0;
 			if ((err = get_controls(card, config)))
-				return err;
+				goto out;
 		}
 	} else {
 		int cardno;
@@ -1532,26 +1604,40 @@ int save_state(const char *file, const char *cardname)
 		cardno = snd_card_get_index(cardname);
 		if (cardno < 0) {
 			error("Cannot find soundcard '%s'...", cardname);
-			return cardno;
+			err = cardno;
+			goto out;
 		}
 		if ((err = get_controls(cardno, config))) {
-			return err;
+			goto out;
 		}
 	}
 	
-	if (stdio)
+	if (stdio) {
 		err = snd_output_stdio_attach(&out, stdout, 0);
-	else
-		err = snd_output_stdio_open(&out, file, "w");
+	} else {
+		err = snd_output_stdio_open(&out, nfile, "w");
+	}
 	if (err < 0) {
 		error("Cannot open %s for writing: %s", file, snd_strerror(err));
-		return -errno;
+		err = -errno;
+		goto out;
 	}
 	err = snd_config_save(config, out);
 	snd_output_close(out);
-	if (err < 0)
+	if (err < 0) {
 		error("snd_config_save: %s", snd_strerror(err));
-	return 0;
+	} else {
+		err = rename(nfile, file);
+		if (err < 0)
+			error("rename failed: %s (%s)", strerror(-err), file);
+	}
+out:
+	if (!stdio)
+		state_lock(file, 0, 10);
+	free(nfile);
+	snd_config_delete(config);
+	snd_config_update_free_global();
+	return err;
 }
 
 int load_state(const char *file, const char *initfile, const char *cardname,
@@ -1560,7 +1646,7 @@ int load_state(const char *file, const char *initfile, const char *cardname,
 	int err, finalerr = 0;
 	snd_config_t *config;
 	snd_input_t *in;
-	int stdio;
+	int stdio, locked = 0;
 
 	err = snd_config_top(&config);
 	if (err < 0) {
@@ -1568,16 +1654,21 @@ int load_state(const char *file, const char *initfile, const char *cardname,
 		return err;
 	}
 	stdio = !strcmp(file, "-");
-	if (stdio)
+	if (stdio) {
 		err = snd_input_stdio_attach(&in, stdin, 0);
-	else
-		err = snd_input_stdio_open(&in, file, "r");
+	} else {
+		err = state_lock(file, 1, 10);
+		locked = err >= 0;
+		err = err >= 0 ? snd_input_stdio_open(&in, file, "r") : err;
+	}
 	if (err >= 0) {
 		err = snd_config_load(config, in);
 		snd_input_close(in);
+		if (locked)
+			state_lock(file, 0, 10);
 		if (err < 0) {
 			error("snd_config_load error: %s", snd_strerror(err));
-			return err;
+			goto out;
 		}
 	} else {
 		int card, first = 1;
@@ -1585,13 +1676,24 @@ int load_state(const char *file, const char *initfile, const char *cardname,
 
 		error("Cannot open %s for reading: %s", file, snd_strerror(err));
 		finalerr = err;
-		card = -1;
+		if (cardname) {
+			card = snd_card_get_index(cardname);
+			if (card < 0) {
+				error("Cannot find soundcard '%s'...", cardname);
+				err = -ENODEV;
+				goto out;
+			}
+			goto single;
+		} else {
+			card = -1;
+		}
 		/* find each installed soundcards */
-		while (1) {
+		while (!cardname) {
 			if (snd_card_next(&card) < 0)
 				break;
 			if (card < 0)
 				break;
+single:
 			first = 0;
 			if (!do_init)
 				break;
@@ -1605,7 +1707,8 @@ int load_state(const char *file, const char *initfile, const char *cardname,
 		}
 		if (first)
 			finalerr = 0;	/* no cards, no error code */
-		return finalerr;
+		err = finalerr;
+		goto out;
 	}
 
 	if (!cardname) {
@@ -1620,10 +1723,12 @@ int load_state(const char *file, const char *initfile, const char *cardname,
 			if (card < 0) {
 				if (first) {
 					if (ignore_nocards) {
-						return 0;
+						err = 0;
+						goto out;
 					} else {
 						error("No soundcards found...");
-						return -ENODEV;
+						err = -ENODEV;
+						goto out;
 					}
 				}
 				break;
@@ -1650,7 +1755,8 @@ int load_state(const char *file, const char *initfile, const char *cardname,
 		cardno = snd_card_get_index(cardname);
 		if (cardno < 0) {
 			error("Cannot find soundcard '%s'...", cardname);
-			return -ENODEV;
+			err = -ENODEV;
+			goto out;
 		}
 		/* do a check if controls matches state file */
 		if (do_init && set_controls(cardno, config, 0)) {
@@ -1663,8 +1769,12 @@ int load_state(const char *file, const char *initfile, const char *cardname,
 		if ((err = set_controls(cardno, config, 1))) {
 			initfailed(cardno, "restore", err);
 			if (!force_restore)
-				return err;
+				goto out;
 		}
 	}
-	return finalerr;
+	err = finalerr;
+out:
+	snd_config_delete(config);
+	snd_config_update_free_global();
+	return err;
 }
